@@ -30,6 +30,12 @@
     #define XSIMD_BROADCAST set_simd
 #endif
 
+#if XSIMD_VERSION_MAJOR >= 9
+    #define XSIMD_REDUCE_ADD reduce_add
+#else
+    #define XSIMD_REDUCE_ADD hadd
+#endif
+
 namespace xs = xsimd;
 
 <%def name="middle_j(kparallel, IBLOCK, JBLOCK)">
@@ -39,7 +45,11 @@ namespace xs = xsimd;
         // setup simd accumulators
         % for ir in range(IBLOCK):
             % for jr in range(JBLOCK):
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+		auto accumsimd${ir}_${jr} = (F)0.0;
+#else
                 auto accumsimd${ir}_${jr} = xs::XSIMD_BROADCAST(((F)0.0));
+#endif
             % endfor
         % endfor
 
@@ -72,10 +82,18 @@ namespace xs = xsimd;
             % endfor
             ) {
             % for ir in range(IBLOCK):
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+		auto Xtd${ir} = *Lptr${ir};
+#else
                 auto Xtd${ir} = xs::load_aligned(Lptr${ir});
+#endif
                 % for jr in range(JBLOCK):
                 {
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+		    auto Xsimd = *Rptr${jr};
+#else
                     auto Xsimd = xs::load_aligned(Rptr${jr});
+#endif
                     accumsimd${ir}_${jr} = xs::fma(Xtd${ir}, Xsimd, accumsimd${ir}_${jr});
                 }
                 % endfor
@@ -85,7 +103,11 @@ namespace xs = xsimd;
         // horizontal sum of the simd blocks
         % for ir in range(IBLOCK):
             % for jr in range(JBLOCK):
-                F accum${ir}_${jr} = xs::hadd(accumsimd${ir}_${jr});
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+		F accum${ir}_${jr} = accumsimd${ir}_${jr};
+#else
+                F accum${ir}_${jr} = xs::XSIMD_REDUCE_ADD(accumsimd${ir}_${jr});
+#endif
             % endfor
         % endfor
 
@@ -137,25 +159,29 @@ namespace xs = xsimd;
 </%def>
 
 <%def name="dense_base_tmpl(kparallel)">
-template <typename F>
+template <typename Int, typename F>
 void dense_base${kparallel}(F* R, F* L, F* d, F* out,
-                int out_m,
-                int imin2, int imax2,
-                int jmin2, int jmax2, 
-                int kmin, int kmax, int innerblock, int kstep) 
+                Py_ssize_t out_m,
+                Py_ssize_t imin2, Py_ssize_t imax2,
+                Py_ssize_t jmin2, Py_ssize_t jmax2, 
+                Py_ssize_t kmin, Py_ssize_t kmax, Int innerblock, Int kstep) 
 {
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+    constexpr std::size_t simd_size = 1;
+#else
     constexpr std::size_t simd_size = xsimd::simd_type<F>::size;
-    for (int imin = imin2; imin < imax2; imin+=innerblock) {
-        int imax = imin + innerblock; 
+#endif
+    for (Py_ssize_t imin = imin2; imin < imax2; imin+=innerblock) {
+        Py_ssize_t imax = imin + innerblock; 
         if (imax > imax2) {
             imax = imax2; 
         }
-        for (int jmin = jmin2; jmin < jmax2; jmin+=innerblock) {
-            int jmax = jmin + innerblock; 
+        for (Py_ssize_t jmin = jmin2; jmin < jmax2; jmin+=innerblock) {
+            Py_ssize_t jmax = jmin + innerblock; 
             if (jmax > jmax2) {
                 jmax = jmax2; 
             }
-            int i = imin;
+            Py_ssize_t i = imin;
             % for IBLOCK in [4, 2, 1]:
             {
                 ${outer_i(kparallel, IBLOCK, [4, 2, 1])}
@@ -172,11 +198,11 @@ ${dense_base_tmpl(False)}
 <%def name="k_loop(kparallel, order)">
 % if kparallel:
     #pragma omp parallel for
-    for (int Rk = 0; Rk < in_n; Rk+=kratio*thresh1d) {
+    for (Py_ssize_t Rk = 0; Rk < in_n; Rk+=kratio*thresh1d) {
 % else:
-    for (int Rk = 0; Rk < in_n; Rk+=kratio*thresh1d) {
+    for (Py_ssize_t Rk = 0; Rk < in_n; Rk+=kratio*thresh1d) {
 % endif
-    int Rkmax2 = Rk + kratio*thresh1d; 
+    int Rkmax2 = Rk + kratio * thresh1d; 
     if (Rkmax2 > in_n) {
         Rkmax2 = in_n; 
     }
@@ -184,50 +210,50 @@ ${dense_base_tmpl(False)}
     F* R = Rglobal.get();
     % if kparallel:
     R += omp_get_thread_num()*thresh1d*thresh1d*kratio*kratio;
-    for (int Cjj = Cj; Cjj < Cjmax2; Cjj++) {
+    for (Py_ssize_t Cjj = Cj; Cjj < Cjmax2; Cjj++) {
     % else:
     #pragma omp parallel for
-    for (int Cjj = Cj; Cjj < Cjmax2; Cjj++) {
+    for (Py_ssize_t Cjj = Cj; Cjj < Cjmax2; Cjj++) {
     % endif
         {
-            int jj = cols[Cjj];
+            Int jj = cols[Cjj];
             %if order == 'F':
                 //TODO: this could use some pointer logic for the R assignment?
-                for (int Rkk=Rk; Rkk<Rkmax2; Rkk++) {
-                    int kk = rows[Rkk];
-                    R[(Cjj-Cj)*kratio*thresh1d+(Rkk-Rk)] = d[kk] * X[jj*n+kk];
+                for (Py_ssize_t Rkk=Rk; Rkk<Rkmax2; Rkk++) {
+                    Int kk = rows[Rkk];
+                    R[(Cjj-Cj) * kratio * thresh1d + (Rkk-Rk)] = d[kk] * X[(Py_ssize_t) jj * n + kk];
                 }
             % else:
-                for (int Rkk=Rk; Rkk<Rkmax2; Rkk++) {
-                    int kk = rows[Rkk];
-                    R[(Cjj-Cj)*kratio*thresh1d+(Rkk-Rk)] = d[kk] * X[kk*m+jj];
+                for (Py_ssize_t Rkk=Rk; Rkk<Rkmax2; Rkk++) {
+                    Int kk = rows[Rkk];
+                    R[(Cjj-Cj) * kratio * thresh1d + (Rkk-Rk)] = d[kk] * X[(Py_ssize_t) kk * m + jj];
                 }
             % endif
         }
     }
 
     % if kparallel:
-        for (int Ci = Cj; Ci < out_m; Ci+=thresh1d) {
+        for (Py_ssize_t Ci = Cj; Ci < out_m; Ci+=thresh1d) {
     % else:
         #pragma omp parallel for
-        for (int Ci = Cj; Ci < out_m; Ci+=thresh1d) {
+        for (Py_ssize_t Ci = Cj; Ci < out_m; Ci+=thresh1d) {
     % endif
-        int Cimax2 = Ci + thresh1d; 
+        Py_ssize_t Cimax2 = Ci + thresh1d; 
         if (Cimax2 > out_m) {
             Cimax2 = out_m; 
         }
         F* L = &Lglobal.get()[omp_get_thread_num()*thresh1d*thresh1d*kratio];
-        for (int Cii = Ci; Cii < Cimax2; Cii++) {
-            int ii = cols[Cii];
+        for (Py_ssize_t Cii = Ci; Cii < Cimax2; Cii++) {
+            Int ii = cols[Cii];
             %if order == 'F':
-                for (int Rkk=Rk; Rkk<Rkmax2; Rkk++) {
-                    int kk = rows[Rkk];
-                    L[(Cii-Ci)*kratio*thresh1d+(Rkk-Rk)] = X[ii*n+kk];
+                for (Py_ssize_t Rkk=Rk; Rkk<Rkmax2; Rkk++) {
+                    Int kk = rows[Rkk];
+                    L[(Py_ssize_t) (Cii-Ci) * kratio * thresh1d + (Rkk-Rk)] = X[(Py_ssize_t) ii * n + kk];
                 }
             % else:
-                for (int Rkk=Rk; Rkk<Rkmax2; Rkk++) {
-                    int kk = rows[Rkk];
-                    L[(Cii-Ci)*kratio*thresh1d+(Rkk-Rk)] = X[kk*m+ii];
+                for (Py_ssize_t Rkk=Rk; Rkk<Rkmax2; Rkk++) {
+                    Int kk = rows[Rkk];
+                    L[(Py_ssize_t) (Cii-Ci) * kratio * thresh1d + (Rkk-Rk)] = X[(Py_ssize_t) kk * m + ii];
                 }
             % endif
         }
@@ -238,15 +264,19 @@ ${dense_base_tmpl(False)}
 
 
 <%def name="dense_sandwich_tmpl(order)">
-template <typename F>
-void _dense${order}_sandwich(int* rows, int* cols, F* X, F* d, F* out,
-        int in_n, int out_m, int m, int n, int thresh1d, int kratio, int innerblock) 
+template <typename Int, typename F>
+void _dense${order}_sandwich(Int* rows, Int* cols, F* X, F* d, F* out,
+        Int in_n, Int out_m, Int m, Int n, Int thresh1d, Int kratio, Int innerblock) 
 {
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+    constexpr std::size_t simd_size = 1;
+#else
     constexpr std::size_t simd_size = xsimd::simd_type<F>::size;
+#endif
     constexpr auto alignment = simd_size * sizeof(F);
 
     bool kparallel = (in_n / (kratio*thresh1d)) > (out_m / thresh1d);
-    size_t Rsize = thresh1d*thresh1d*kratio*kratio;
+    Py_ssize_t Rsize = thresh1d*thresh1d*kratio*kratio;
     if (kparallel) {
         Rsize *= omp_get_max_threads();
     }
@@ -256,8 +286,8 @@ void _dense${order}_sandwich(int* rows, int* cols, F* X, F* d, F* out,
         omp_get_max_threads() * thresh1d * thresh1d * kratio, 
         alignment
     );
-    for (int Cj = 0; Cj < out_m; Cj+=kratio*thresh1d) {
-        int Cjmax2 = Cj + kratio*thresh1d; 
+    for (Py_ssize_t Cj = 0; Cj < out_m; Cj+=kratio*thresh1d) {
+        Py_ssize_t Cjmax2 = Cj + kratio*thresh1d; 
         if (Cjmax2 > out_m) {
             Cjmax2 = out_m; 
         }
@@ -269,8 +299,8 @@ void _dense${order}_sandwich(int* rows, int* cols, F* X, F* d, F* out,
     }
 
     #pragma omp parallel for if(out_m > 100)
-    for (int Ci = 0; Ci < out_m; Ci++) {
-        for (int Cj = 0; Cj <= Ci; Cj++) {
+    for (Py_ssize_t Ci = 0; Ci < out_m; Ci++) {
+        for (Py_ssize_t Cj = 0; Cj <= Ci; Cj++) {
             out[Cj * out_m + Ci] = out[Ci * out_m + Cj];
         }
     }
@@ -282,11 +312,15 @@ ${dense_sandwich_tmpl('F')}
 
 
 <%def name="dense_rmatvec_tmpl(order)">
-template <typename F>
-void _dense${order}_rmatvec(int* rows, int* cols, F* X, F* v, F* out,
-        int n_rows, int n_cols, int m, int n) 
+template <typename Int, typename F>
+void _dense${order}_rmatvec(Int* rows, Int* cols, F* X, F* v, F* out,
+        Int n_rows, Int n_cols, Int m, Int n) 
 {
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+    constexpr std::size_t simd_size = 1;
+#else
     constexpr std::size_t simd_size = xsimd::simd_type<F>::size;
+#endif
     constexpr std::size_t alignment = simd_size * sizeof(F);
 
     auto outglobal = make_aligned_unique<F>(omp_get_max_threads()*n_cols, alignment);
@@ -295,27 +329,27 @@ void _dense${order}_rmatvec(int* rows, int* cols, F* X, F* v, F* out,
     constexpr int colblocksize = 4;
 
     #pragma omp parallel for
-    for (int Ci = 0; Ci < n_rows; Ci += rowblocksize) {
-        int Cimax = Ci + rowblocksize;
+    for (Py_ssize_t Ci = 0; Ci < n_rows; Ci += rowblocksize) {
+        Py_ssize_t Cimax = Ci + rowblocksize;
         if (Cimax > n_rows) {
             Cimax = n_rows;
         }
 
         F* outlocal = &outglobal.get()[omp_get_thread_num()*n_cols];
 
-        for (int Cj = 0; Cj < n_cols; Cj += colblocksize) {
-            int Cjmax = Cj + colblocksize;
+        for (Py_ssize_t Cj = 0; Cj < n_cols; Cj += colblocksize) {
+            Py_ssize_t Cjmax = Cj + colblocksize;
             if (Cjmax > n_cols) {
                 Cjmax = n_cols;
             }
 
             % if order == 'F':
-                for (int Cjj = Cj; Cjj < Cjmax; Cjj++) {
-                    int j = cols[Cjj];
+                for (Py_ssize_t Cjj = Cj; Cjj < Cjmax; Cjj++) {
+                    Int j = cols[Cjj];
                     F out_entry = 0.0;
-                    for (int Cii = Ci; Cii < Cimax; Cii++) {
-                        int i = rows[Cii];
-                        F Xv = X[j * n + i];
+                    for (Py_ssize_t Cii = Ci; Cii < Cimax; Cii++) {
+                        Int i = rows[Cii];
+                        F Xv = X[(Py_ssize_t) j * n + i];
                         F vv = v[i];
                         out_entry += Xv * vv;
                     }
@@ -323,22 +357,22 @@ void _dense${order}_rmatvec(int* rows, int* cols, F* X, F* v, F* out,
                     outlocal[Cjj] = out_entry;
                 }
             % else:
-                for (int Cjj = Cj; Cjj < Cjmax; Cjj++) {
+                for (Py_ssize_t Cjj = Cj; Cjj < Cjmax; Cjj++) {
                     outlocal[Cjj] = 0.0;
                 }
-                for (int Cii = Ci; Cii < Cimax; Cii++) {
-                    int i = rows[Cii];
+                for (Py_ssize_t Cii = Ci; Cii < Cimax; Cii++) {
+                    Int i = rows[Cii];
                     F vv = v[i];
-                    for (int Cjj = Cj; Cjj < Cjmax; Cjj++) {
-                        int j = cols[Cjj];
-                        F Xv = X[i * m + j];
+                    for (Py_ssize_t Cjj = Cj; Cjj < Cjmax; Cjj++) {
+                        Int j = cols[Cjj];
+                        F Xv = X[(Py_ssize_t) i * m + j];
                         outlocal[Cjj] += Xv * vv;
                     }
                 }
             % endif
         }
 
-        for (int Cj = 0; Cj < n_cols; Cj++) {
+        for (Py_ssize_t Cj = 0; Cj < n_cols; Cj++) {
             #pragma omp atomic
             out[Cj] += outlocal[Cj];
         }
@@ -349,28 +383,28 @@ ${dense_rmatvec_tmpl('C')}
 ${dense_rmatvec_tmpl('F')}
 
 <%def name="dense_matvec_tmpl(order)">
-template <typename F>
-void _dense${order}_matvec(int* rows, int* cols, F* X, F* v, F* out,
-        int n_rows, int n_cols, int m, int n) 
+template <typename Int, typename F>
+void _dense${order}_matvec(Int* rows, Int* cols, F* X, F* v, F* out,
+        Int n_rows, Int n_cols, Int m, Int n) 
 {
     constexpr int rowblocksize = 256;
 
     #pragma omp parallel for
-    for (int Ci = 0; Ci < n_rows; Ci += rowblocksize) {
-        int Cimax = Ci + rowblocksize;
+    for (Py_ssize_t Ci = 0; Ci < n_rows; Ci += rowblocksize) {
+        Int Cimax = Ci + rowblocksize;
         if (Cimax > n_rows) {
             Cimax = n_rows;
         }
-        for (int Cii = Ci; Cii < Cimax; Cii++) {
+        for (Py_ssize_t Cii = Ci; Cii < Cimax; Cii++) {
             F out_entry = 0.0;
-            int i = rows[Cii];
-            for (int Cjj = 0; Cjj < n_cols; Cjj++) {
-                int j = cols[Cjj];
+            Int i = rows[Cii];
+            for (Py_ssize_t Cjj = 0; Cjj < n_cols; Cjj++) {
+                Int j = cols[Cjj];
                 F vv = v[j];
                 % if order == 'F':
-                    F Xv = X[j * n + i];
+                    F Xv = X[(Py_ssize_t) j * n + i];
                 % else:
-                    F Xv = X[i * m + j];
+                    F Xv = X[(Py_ssize_t) i * m + j];
                 % endif
                 out_entry += Xv * vv;
             }
