@@ -6,6 +6,12 @@
 
 #include "alloc.h"
 
+#ifdef _WIN32
+    #define SIZE_T long long
+#else
+    #define SIZE_T size_t
+#endif
+
 #if XSIMD_VERSION_MAJOR >= 8
     #define XSIMD_BROADCAST broadcast
 #else
@@ -24,11 +30,16 @@ void _csr_dense${order}_sandwich(
     Int nrows, Int nA_cols, Int nB_cols
     ) 
 {
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+    constexpr Int simd_size = 1;
+#else
     constexpr Int simd_size = xsimd::simd_type<F>::size;
+#endif
+
     constexpr auto alignment = simd_size*sizeof(F);
 
-    Int kblock = 128;
-    Int jblock = 128;
+    int kblock = 128;
+    int jblock = 128;
     auto Rglobal = make_aligned_unique<F>(
         omp_get_max_threads() * kblock * jblock,
         alignment
@@ -36,83 +47,95 @@ void _csr_dense${order}_sandwich(
 
     std::vector<Int> Acol_map(m, -1);
     // Don't parallelize because the number of columns is small
-    for (Int Ci = 0; Ci < nA_cols; Ci++) {
-        Int i = A_cols[Ci];
-        Acol_map[i] = Ci;
+    for (Py_ssize_t Ci = 0; Ci < nA_cols; Ci++) {
+        Acol_map[A_cols[Ci]] = Ci;
     }
 
     #pragma omp parallel
     {
-        Int nB_cols_rounded = ceil(((float)nB_cols) / ((float)simd_size)) * simd_size;
+        Py_ssize_t nB_cols_rounded = ceil(((float)nB_cols) / ((float)simd_size)) * simd_size;
         auto outtemp = make_aligned_unique<F>(
             nA_cols * nB_cols_rounded,
             alignment
         );
-        for (Int Ci = 0; Ci < nA_cols; Ci++) {
-            for (Int Cj = 0; Cj < nB_cols; Cj++) {
-                outtemp.get()[Ci*nB_cols_rounded+Cj] = 0.0;
+        for (Py_ssize_t Ci = 0; Ci < nA_cols; Ci++) {
+            for (Py_ssize_t Cj = 0; Cj < nB_cols; Cj++) {
+                outtemp.get()[Ci * nB_cols_rounded + Cj] = 0.0;
             }
         }
 
         #pragma omp for
-        for (Int Ckk = 0; Ckk < nrows; Ckk+=kblock) {
-            Int Ckmax = Ckk + kblock;
+        for (Py_ssize_t Ckk = 0; Ckk < nrows; Ckk+=kblock) {
+            Py_ssize_t Ckmax = Ckk + kblock;
             if (Ckmax > nrows) {
                 Ckmax = nrows;
             }
-            for (Int Cjj = 0; Cjj < nB_cols; Cjj+=jblock) {
-                Int Cjmax = Cjj + jblock;
+            for (Py_ssize_t Cjj = 0; Cjj < nB_cols; Cjj+=jblock) {
+                Py_ssize_t Cjmax = Cjj + jblock;
                 if (Cjmax > nB_cols) {
                     Cjmax = nB_cols;
                 }
 
                 F* R = &Rglobal.get()[omp_get_thread_num()*kblock*jblock];
-                for (Int Ck = Ckk; Ck < Ckmax; Ck++) {
+                for (Py_ssize_t Ck = Ckk; Ck < Ckmax; Ck++) {
                     Int k = rows[Ck];
-                    for (Int Cj = Cjj; Cj < Cjmax; Cj++) {
+                    for (Py_ssize_t Cj = Cjj; Cj < Cjmax; Cj++) {
                         Int j = B_cols[Cj];
                         %if order == 'C':
-                            F Bv = B[k * r + j];
+                            F Bv = B[(Py_ssize_t) k * r + j];
                         % else:
-                            F Bv = B[j * n + k];
+                            F Bv = B[(Py_ssize_t) j * n + k];
                         % endif
-                        R[(Ck-Ckk) * jblock + (Cj-Cjj)] = d[k] * Bv;
+                        R[(Py_ssize_t) (Ck-Ckk) * jblock + (Cj-Cjj)] = d[k] * Bv;
                     }
                 }
 
-                for (Int Ck = Ckk; Ck < Ckmax; Ck++) {
+                for (Py_ssize_t Ck = Ckk; Ck < Ckmax; Ck++) {
                     Int k = rows[Ck];
                     for (Int A_idx = Aindptr[k]; A_idx < Aindptr[k+1]; A_idx++) {
                         Int i = Aindices[A_idx];
-                        Int Ci = Acol_map[i];
+                        Py_ssize_t Ci = Acol_map[i];
                         if (Ci == -1) {
                             continue;
                         }
 
                         F Q = Adata[A_idx];
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+			auto Qsimd = Q;
+#else
                         auto Qsimd = xs::XSIMD_BROADCAST(Q);
+#endif
 
-                        Int Cj = Cjj;
-                        Int Cjmax2 = Cjj + ((Cjmax - Cjj) / simd_size) * simd_size;
+                        Py_ssize_t Cj = Cjj;
+                        Py_ssize_t Cjmax2 = Cjj + ((Cjmax - Cjj) / simd_size) * simd_size;
                         for (; Cj < Cjmax2; Cj+=simd_size) {
-                            auto Bsimd = xs::load_aligned(&R[(Ck-Ckk)*jblock+(Cj-Cjj)]);
-                            auto outsimd = xs::load_aligned(&outtemp.get()[Ci*nB_cols_rounded+Cj]);
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+                            auto Bsimd = R[(Py_ssize_t) (Ck-Ckk) * jblock + (Cj-Cjj)];
+                            auto outsimd = outtemp.get()[Ci * nB_cols_rounded + Cj];
+#else
+                            auto Bsimd = xs::load_aligned(&R[(Py_ssize_t) (Ck-Ckk) * jblock + (Cj-Cjj)]);
+                            auto outsimd = xs::load_aligned(&outtemp.get()[Ci * nB_cols_rounded + Cj]);
+#endif
                             outsimd = xs::fma(Qsimd, Bsimd, outsimd);
-                            outsimd.store_aligned(&outtemp.get()[Ci*nB_cols_rounded+Cj]);
+#ifdef XSIMD_NO_SUPPORTED_ARCHITECTURE
+                            outtemp.get()[Ci * nB_cols_rounded + Cj] = outsimd;
+#else
+                            outsimd.store_aligned(&outtemp.get()[Ci * nB_cols_rounded + Cj]);
+#endif
                         }
 
                         for (; Cj < Cjmax; Cj++) {
-                            outtemp.get()[Ci*nB_cols_rounded+Cj] += Q * R[(Ck-Ckk)*jblock+(Cj-Cjj)];
+                            outtemp.get()[Ci * nB_cols_rounded + Cj] += Q * R[(Py_ssize_t) (Ck-Ckk) * jblock + (Cj-Cjj)];
                         }
                     }
                 }
             }
         }
 
-        for (Int Ci = 0; Ci < nA_cols; Ci++) {
-            for (Int Cj = 0; Cj < nB_cols; Cj++) {
+        for (Py_ssize_t Ci = 0; Ci < nA_cols; Ci++) {
+            for (Py_ssize_t Cj = 0; Cj < nB_cols; Cj++) {
                 #pragma omp atomic
-                out[Ci*nB_cols+Cj] += outtemp.get()[Ci*nB_cols_rounded+Cj];
+                out[(Py_ssize_t) Ci * nB_cols + Cj] += outtemp.get()[Ci * nB_cols_rounded + Cj];
             }
         }
     }
