@@ -1,11 +1,13 @@
-from typing import List, Optional, Union
+from typing import Optional, Union
 
 import numpy as np
+import pandas as pd
 import pytest
 import scipy.sparse as sps
 
 import tabmat as tm
-from tabmat.constructor import _split_sparse_and_dense_parts
+from tabmat import from_pandas
+from tabmat.constructor_util import _split_sparse_and_dense_parts
 from tabmat.dense_matrix import DenseMatrix
 from tabmat.ext.sparse import csr_dense_sandwich
 from tabmat.split_matrix import SplitMatrix
@@ -59,27 +61,32 @@ def split_mat() -> SplitMatrix:
     return mat
 
 
-def get_split_with_cat_components() -> List[
-    Union[tm.SparseMatrix, tm.DenseMatrix, tm.CategoricalMatrix]
-]:
+def get_split_with_cat_components(
+    missing,
+) -> list[Union[tm.SparseMatrix, tm.DenseMatrix, tm.CategoricalMatrix]]:
     n_rows = 10
     np.random.seed(0)
     dense_1 = tm.DenseMatrix(np.random.random((n_rows, 3)))
     sparse_1 = tm.SparseMatrix(sps.random(n_rows, 3).tocsc())
-    cat = tm.CategoricalMatrix(np.random.choice(range(3), n_rows))
+    if missing:
+        cat = tm.CategoricalMatrix(
+            np.random.choice([0, 1, 2, None], n_rows), cat_missing_method="zero"
+        )
+    else:
+        cat = tm.CategoricalMatrix(np.random.choice(range(3), n_rows))
     dense_2 = tm.DenseMatrix(np.random.random((n_rows, 3)))
     sparse_2 = tm.SparseMatrix(sps.random(n_rows, 3, density=0.5).tocsc())
-    cat_2 = tm.CategoricalMatrix(np.random.choice(range(3), n_rows))
+    cat_2 = tm.CategoricalMatrix(np.random.choice(range(3), n_rows), drop_first=True)
     return [dense_1, sparse_1, cat, dense_2, sparse_2, cat_2]
 
 
-def split_with_cat() -> SplitMatrix:
+def split_with_cat(missing) -> SplitMatrix:
     """Initialized with multiple sparse and dense parts and no indices."""
-    return tm.SplitMatrix(get_split_with_cat_components())
+    return tm.SplitMatrix(get_split_with_cat_components(missing))
 
 
-def split_with_cat_64() -> SplitMatrix:
-    mat = tm.SplitMatrix(get_split_with_cat_components())
+def split_with_cat_64(missing) -> SplitMatrix:
+    mat = tm.SplitMatrix(get_split_with_cat_components(missing))
     matrices = mat.matrices
 
     for i, mat_ in enumerate(mat.matrices):
@@ -97,7 +104,15 @@ def split_with_cat_64() -> SplitMatrix:
     return tm.SplitMatrix(matrices, mat.indices)
 
 
-@pytest.mark.parametrize("mat", [split_with_cat(), split_with_cat_64()])
+@pytest.mark.parametrize(
+    "mat",
+    [
+        split_with_cat(False),
+        split_with_cat_64(False),
+        split_with_cat(True),
+        split_with_cat_64(True),
+    ],
+)
 def test_init(mat: SplitMatrix):
     assert len(mat.indices) == 4
     assert len(mat.matrices) == 4
@@ -107,6 +122,22 @@ def test_init(mat: SplitMatrix):
     assert mat.matrices[2].shape == (10, 3)
 
 
+@pytest.mark.parametrize(
+    "mat",
+    [
+        split_with_cat(False),
+        split_with_cat_64(False),
+        split_with_cat(True),
+        split_with_cat_64(True),
+    ],
+)
+def test_init_from_split(mat):
+    np.testing.assert_array_equal(mat.toarray(), tm.SplitMatrix([mat]).toarray())
+    np.testing.assert_array_equal(
+        np.hstack([mat.toarray(), mat.toarray()]), tm.SplitMatrix([mat, mat]).toarray()
+    )
+
+
 def test_init_unsorted_indices():
     dense = tm.DenseMatrix(np.random.random((10, 3)))
     with pytest.raises(ValueError):
@@ -131,12 +162,20 @@ def test_sandwich_sparse_dense(X: np.ndarray, Acols, Bcols):
     A = sps.random(n, 2).tocsr()
     rows = np.arange(d.shape[0], dtype=np.int32)
     result = csr_dense_sandwich(A, X, d, rows, Acols, Bcols)
-    expected = A.T.A[Acols, :] @ np.diag(d) @ X[:, Bcols]
+    expected = A.T.toarray()[Acols, :] @ np.diag(d) @ X[:, Bcols]
     np.testing.assert_allclose(result, expected)
 
 
 # TODO: ensure cols are in order
-@pytest.mark.parametrize("mat", [split_mat(), split_with_cat(), split_with_cat_64()])
+@pytest.mark.parametrize(
+    "mat",
+    [
+        split_with_cat(False),
+        split_with_cat_64(False),
+        split_with_cat(True),
+        split_with_cat_64(True),
+    ],
+)
 @pytest.mark.parametrize(
     "cols",
     [None, [0], [1, 2, 3], [1, 5]],
@@ -145,18 +184,26 @@ def test_sandwich(mat: tm.SplitMatrix, cols):
     for _ in range(10):
         v = np.random.rand(mat.shape[0])
         y1 = mat.sandwich(v, cols=cols)
-        mat_limited = mat.A if cols is None else mat.A[:, cols]
+        mat_limited = mat.toarray() if cols is None else mat.toarray()[:, cols]
         y2 = (mat_limited.T * v[None, :]) @ mat_limited
         np.testing.assert_allclose(y1, y2, atol=1e-12)
 
 
-@pytest.mark.parametrize("mat", [split_mat(), split_with_cat(), split_with_cat_64()])
+@pytest.mark.parametrize(
+    "mat",
+    [
+        split_with_cat(False),
+        split_with_cat_64(False),
+        split_with_cat(True),
+        split_with_cat_64(True),
+    ],
+)
 @pytest.mark.parametrize("cols", [None, [0], [1, 2, 3], [1, 5]])
 def test_split_col_subsets(mat: tm.SplitMatrix, cols):
     subset_cols_indices, subset_cols, n_cols = mat._split_col_subsets(cols)
     n_cols_correct = mat.shape[1] if cols is None else len(cols)
 
-    def _get_lengths(vec_list: List[Optional[np.ndarray]]):
+    def _get_lengths(vec_list: list[Optional[np.ndarray]]):
         return (
             mat_.shape[1] if v is None else len(v)
             for v, mat_ in zip(vec_list, mat.matrices)
@@ -179,56 +226,66 @@ def test_split_col_subsets(mat: tm.SplitMatrix, cols):
             assert (mat.indices[i] == subset_cols_indices[i]).all()
 
 
-def random_split_matrix(seed=0, n_rows=10, n_cols_per=3):
+def random_split_matrix(seed=0, n_rows=10, n_cols_per=3, missing=False):
     if seed is not None:
         np.random.seed(seed)
     dense_1 = tm.DenseMatrix(np.random.random((n_rows, n_cols_per)))
     sparse = tm.SparseMatrix(sps.random(n_rows, n_cols_per).tocsc())
-    cat = tm.CategoricalMatrix(np.random.choice(range(n_cols_per), n_rows))
+    if missing:
+        cat = tm.CategoricalMatrix(
+            np.random.choice(list(range(n_cols_per)) + [None], n_rows),
+            cat_missing_method="zero",
+        )
+    else:
+        cat = tm.CategoricalMatrix(np.random.choice(range(n_cols_per), n_rows))
     dense_2 = tm.DenseMatrix(np.random.random((n_rows, n_cols_per)))
     cat_2 = tm.CategoricalMatrix(np.random.choice(range(n_cols_per), n_rows))
     mat = tm.SplitMatrix([dense_1, sparse, cat, dense_2, cat_2])
     return mat
 
 
-def many_random_tests(checker):
+def many_random_tests(checker, missing):
     for i in range(10):
         mat = random_split_matrix(
             seed=(1 if i == 0 else None),
             n_rows=np.random.randint(130),
             n_cols_per=1 + np.random.randint(10),
+            missing=missing,
         )
         checker(mat)
 
 
-def test_sandwich_many_types():
+@pytest.mark.parametrize("missing", [False, True], ids=["no_missing", "missing"])
+def test_sandwich_many_types(missing):
     def check(mat):
         d = np.random.random(mat.shape[0])
         res = mat.sandwich(d)
-        expected = (mat.A.T * d[None, :]) @ mat.A
+        expected = (mat.toarray().T * d[None, :]) @ mat.toarray()
         np.testing.assert_allclose(res, expected)
 
-    many_random_tests(check)
+    many_random_tests(check, missing)
 
 
-def test_transpose_matvec_many_types():
+@pytest.mark.parametrize("missing", [False, True], ids=["no_missing", "missing"])
+def test_transpose_matvec_many_types(missing):
     def check(mat):
         d = np.random.random(mat.shape[0])
         res = mat.transpose_matvec(d)
-        expected = mat.A.T.dot(d)
+        expected = mat.toarray().T.dot(d)
         np.testing.assert_almost_equal(res, expected)
 
-    many_random_tests(check)
+    many_random_tests(check, missing)
 
 
-def test_matvec_many_types():
+@pytest.mark.parametrize("missing", [False, True], ids=["no_missing", "missing"])
+def test_matvec_many_types(missing):
     def check(mat):
         d = np.random.random(mat.shape[1])
         res = mat.matvec(d)
-        expected = mat.A.dot(d)
+        expected = mat.toarray().dot(d)
         np.testing.assert_almost_equal(res, expected)
 
-    many_random_tests(check)
+    many_random_tests(check, missing)
 
 
 def test_init_from_1d():
@@ -237,3 +294,15 @@ def test_init_from_1d():
 
     res = SplitMatrix([m1, m2])
     assert res.shape == (10, 3)
+
+
+@pytest.mark.parametrize("n_rows", [5, 10, 25])
+def test_matvec(n_rows):
+    np.random.seed(1234)
+    n_cols = 2
+    categories = [f"cat_{val}" for val in range(5)]
+    X = pd.DataFrame(np.random.choice(categories, size=(n_rows, n_cols))).astype(
+        "category"
+    )
+    mat = from_pandas(X, cat_threshold=0)
+    np.testing.assert_allclose(mat.matvec(np.array(mat.shape[1] * [1])), n_cols)
